from django.dispatch import receiver
from django.conf import settings
from django.dispatch.dispatcher import Signal

from .tasks import VkontateUsersFetchUsers

users_to_fetch = Signal(providing_args=['ids'])

@receiver(users_to_fetch)
def fetch_users(ids, **kwargs):
    only_expired = getattr(settings, 'VKONTAKTE_USERS_FETCH_ONLY_EXPIRED_USERS', True)
    async = getattr(settings, 'VKONTAKTE_USERS_FETCH_USERS_ASYNC', False)

    params = dict(ids=ids, only_expired=only_expired)
    if async:
        VkontateUsersFetchUsers.apply_async(kwargs=params)
    else:
        VkontateUsersFetchUsers.apply(kwargs=params)



"""
QuickDjangoTest module for testing in Travis CI https://travis-ci.org
Changes log:
 * 2014-10-24 updated for compatibility with Django 1.7
 * 2014-11-03 different databases support: sqlite3, mysql, postgres
 * 2014-12-31 pep8, python 3 compatibility
 * 2015-02-01 Django 1.9 compatibility
 * 2015-02-25 updated code style
 * 2015-02-26 updated get_database() for Django 1.8
 * 2015-02-27 clean up variables
"""

import argparse
import os
import sys

from django.conf import settings


class QuickDjangoTest(object):

    """
    A quick way to run the Django test suite without a fully-configured project.

    Example usage:

        >>> QuickDjangoTest('app1', 'app2')

    Based on a script published by Lukasz Dziedzia at:
    http://stackoverflow.com/questions/3841725/how-to-launch-tests-for-django-reusable-app
    """
    DIRNAME = os.path.dirname(__file__)
    INSTALLED_APPS = (
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.admin',
    )

    def __init__(self, *args):
        self.apps = args

        # Call the appropriate one
        method = getattr(self, '_tests_%s' % self.version.replace('.', '_'), None)
        if method and callable(method):
            method()
        else:
            self._tests_old()

    @property
    def version(self):
        """
        Figure out which version of Django's test suite we have to play with.
        """
        from django import VERSION
        if VERSION[0] == 1 and VERSION[1] >= 8:
            return '1.8'
        elif VERSION[0] == 1 and VERSION[1] >= 7:
            return '1.7'
        elif VERSION[0] == 1 and VERSION[1] >= 2:
            return '1.2'
        else:
            return

    def get_database(self, version):
        test_db = os.environ.get('DB', 'sqlite')
        if test_db == 'mysql':
            database = {
                'ENGINE': 'django.db.backends.mysql',
                'NAME': 'django',
                'USER': 'root',
            }
        elif test_db == 'postgres':
            database = {
                'ENGINE': 'django.db.backends.postgresql_psycopg2',
                'USER': 'postgres',
                'NAME': 'django',
            }
            if version < 1.8:
                database['OPTIONS'] = {'autocommit': True}
        else:
            database = {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': os.path.join(self.DIRNAME, 'database.db'),
                'USER': '',
                'PASSWORD': '',
                'HOST': '',
                'PORT': '',
            }
        return {'default': database}

    @property
    def custom_settings(self):
        """
        Return custom settings from settings_test.py file
        :return: dict
        """
        try:
            import settings_test
            test_settings = dict([(k, v) for k, v in settings_test.__dict__.items() if k[0] != '_'])
        except ImportError:
            test_settings = {'INSTALLED_APPS': []}
        return test_settings

    def _tests_old(self):
        """
        Fire up the Django test suite from before version 1.2
        """
        test_settings = self.custom_settings
        installed_apps = test_settings.pop('INSTALLED_APPS', ())
        settings.configure(
                DEBUG=True,
                DATABASE_ENGINE='sqlite3',
                DATABASE_NAME=os.path.join(self.DIRNAME, 'database.db'),
                INSTALLED_APPS=tuple(self.INSTALLED_APPS + installed_apps + self.apps),
                **test_settings
        )
        from django.test.simple import run_tests
        failures = run_tests(self.apps, verbosity=1)
        if failures:
            sys.exit(failures)

    def _tests_1_2(self):
        """
        Fire up the Django test suite developed for version 1.2 and up
        """
        test_settings = self.custom_settings
        installed_apps = test_settings.pop('INSTALLED_APPS', ())
        settings.configure(
            DEBUG=True,
            DATABASES=self.get_database(1.2),
            INSTALLED_APPS=tuple(self.INSTALLED_APPS + installed_apps + self.apps),
            **test_settings
        )
        from django.test.simple import DjangoTestSuiteRunner
        failures = DjangoTestSuiteRunner().run_tests(self.apps, verbosity=1)
        if failures:
            sys.exit(failures)

    def _tests_1_7(self):
        """
        Fire up the Django test suite developed for version 1.7 and up
        """
        test_settings = self.custom_settings
        installed_apps = test_settings.pop('INSTALLED_APPS', ())
        settings.configure(
            DEBUG=True,
            DATABASES=self.get_database(1.7),
            MIDDLEWARE_CLASSES=('django.middleware.common.CommonMiddleware',
                                'django.middleware.csrf.CsrfViewMiddleware'),
            INSTALLED_APPS=tuple(self.INSTALLED_APPS + installed_apps + self.apps),
            **test_settings
        )
        from django.test.simple import DjangoTestSuiteRunner
        import django
        django.setup()
        failures = DjangoTestSuiteRunner().run_tests(self.apps, verbosity=1)
        if failures:
            sys.exit(failures)

    def _tests_1_8(self):
        """
        Fire up the Django test suite developed for version 1.8 and up
        """
        test_settings = self.custom_settings
        installed_apps = test_settings.pop('INSTALLED_APPS', ())
        settings.configure(
            DEBUG=True,
            DATABASES=self.get_database(1.8),
            MIDDLEWARE_CLASSES=('django.middleware.common.CommonMiddleware',
                                'django.middleware.csrf.CsrfViewMiddleware'),
            INSTALLED_APPS=tuple(self.INSTALLED_APPS + installed_apps + self.apps),
            **test_settings
        )
        from django.test.runner import DiscoverRunner
        import django
        django.setup()
        failures = DiscoverRunner().run_tests(self.apps, verbosity=1)
        if failures:
            sys.exit(failures)


if __name__ == '__main__':
    """
    What do when the user hits this file from the shell.

    Example usage:

        $ python quicktest.py app1 app2

    """
    parser = argparse.ArgumentParser(
        usage="[args]",
        description="Run Django tests on the provided applications."
    )
    parser.add_argument('apps', nargs='+', type=str)
    QuickDjangoTest(*parser.parse_args().apps)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
import vkontakte_api.models


class Migration(migrations.Migration):

    dependencies = [
        ('vkontakte_places', '__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('fetched', models.DateTimeField(db_index=True, null=True, verbose_name='\u041e\u0431\u043d\u043e\u0432\u043b\u0435\u043d\u043e', blank=True)),
                ('remote_id', models.BigIntegerField(help_text='\u0423\u043d\u0438\u043a\u0430\u043b\u044c\u043d\u044b\u0439 \u0438\u0434\u0435\u043d\u0442\u0438\u0444\u0438\u043a\u0430\u0442\u043e\u0440', serialize=False, verbose_name='ID', primary_key=True)),
                ('first_name', models.CharField(max_length=200)),
                ('last_name', models.CharField(max_length=200)),
                ('screen_name', models.CharField(max_length=100, db_index=True)),
                ('sex', models.PositiveSmallIntegerField(db_index=True, null=True, choices=[(0, '\u043d\u0435 \u0443\u043a.'), (1, '\u0436\u0435\u043d.'), (2, '\u043c\u0443\u0436.')])),
                ('age', models.PositiveSmallIntegerField(null=True, db_index=True)),
                ('timezone', models.IntegerField(null=True)),
                ('rate', models.PositiveIntegerField(null=True, db_index=True)),
                ('bdate', models.CharField(max_length=100)),
                ('activity', models.TextField()),
                ('relation', models.SmallIntegerField(db_index=True, null=True, choices=[(1, '\u041d\u0435 \u0436\u0435\u043d\u0430\u0442 / \u0437\u0430\u043c\u0443\u0436\u0435\u043c'), (2, '\u0415\u0441\u0442\u044c \u0434\u0440\u0443\u0433 / \u043f\u043e\u0434\u0440\u0443\u0433\u0430'), (3, '\u041f\u043e\u043c\u043e\u043b\u0432\u043b\u0435\u043d / \u043f\u043e\u043c\u043e\u043b\u0432\u043b\u0435\u043d\u0430'), (4, '\u0416\u0435\u043d\u0430\u0442 / \u0437\u0430\u043c\u0443\u0436\u0435\u043c'), (5, '\u0412\u0441\u0451 \u0441\u043b\u043e\u0436\u043d\u043e'), (6, '\u0412 \u0430\u043a\u0442\u0438\u0432\u043d\u043e\u043c \u043f\u043e\u0438\u0441\u043a\u0435'), (7, '\u0412\u043b\u044e\u0431\u043b\u0451\u043d / \u0432\u043b\u044e\u0431\u043b\u0435\u043d\u0430')])),
                ('wall_comments', models.NullBooleanField()),
                ('graduation', models.PositiveIntegerField(null=True, verbose_name='\u0414\u0430\u0442\u0430 \u043e\u043a\u043e\u043d\u0447\u0430\u043d\u0438\u044f \u0432\u0443\u0437\u0430')),
                ('university', models.PositiveIntegerField(null=True)),
                ('university_name', models.CharField(max_length=500)),
                ('faculty', models.PositiveIntegerField(null=True)),
                ('faculty_name', models.CharField(max_length=500)),
                ('has_mobile', models.NullBooleanField(db_index=True)),
                ('home_phone', models.CharField(max_length=50)),
                ('mobile_phone', models.CharField(max_length=50)),
                ('photo', models.URLField()),
                ('photo_big', models.URLField()),
                ('photo_medium', models.URLField()),
                ('photo_medium_rec', models.URLField()),
                ('photo_rec', models.URLField()),
                ('twitter', models.CharField(max_length=500)),
                ('facebook', models.CharField(max_length=500)),
                ('facebook_name', models.CharField(max_length=500)),
                ('skype', models.CharField(max_length=500)),
                ('livejournal', models.CharField(max_length=500)),
                ('interests', models.TextField()),
                ('movies', models.TextField()),
                ('tv', models.TextField()),
                ('books', models.TextField()),
                ('games', models.TextField()),
                ('about', models.TextField()),
                ('friends_count', models.PositiveIntegerField(default=0, verbose_name='\u0414\u0440\u0443\u0437\u0435\u0439')),
                ('counters_updated', models.DateTimeField(help_text='\u0421\u0447\u0435\u0442\u0447\u0438\u043a\u0438 \u0431\u044b\u043b\u0438 \u043e\u0431\u043d\u043e\u0432\u043b\u0435\u043d\u044b', null=True, db_index=True)),
                ('sum_counters', models.PositiveIntegerField(default=0, help_text='\u0421\u0443\u043c\u043c\u0430 \u0432\u0441\u0435\u0445 \u0441\u0447\u0435\u0442\u0447\u0438\u043a\u043e\u0432')),
                ('albums', models.PositiveIntegerField(default=0, verbose_name='\u0424\u043e\u0442\u043e\u0430\u043b\u044c\u0431\u043e\u043c\u043e\u0432')),
                ('videos', models.PositiveIntegerField(default=0, verbose_name='\u0412\u0438\u0434\u0435\u043e\u0437\u0430\u043f\u0438\u0441\u0435\u0439')),
                ('audios', models.PositiveIntegerField(default=0, verbose_name='\u0410\u0443\u0434\u0438\u043e\u0437\u0430\u043f\u0438\u0441\u0435\u0439')),
                ('followers', models.PositiveIntegerField(default=0, verbose_name='\u041f\u043e\u0434\u043f\u0438\u0441\u0447\u0438\u043a\u043e\u0432')),
                ('friends', models.PositiveIntegerField(default=0, verbose_name='\u0414\u0440\u0443\u0437\u0435\u0439', db_index=True)),
                ('mutual_friends', models.PositiveIntegerField(default=0, verbose_name='\u041e\u0431\u0449\u0438\u0445 \u0434\u0440\u0443\u0437\u0435\u0439')),
                ('notes', models.PositiveIntegerField(default=0, verbose_name='\u0417\u0430\u043c\u0435\u0442\u043e\u043a')),
                ('subscriptions', models.PositiveIntegerField(default=0, verbose_name='\u041f\u043e\u0434\u043f\u0438\u0441\u043e\u043a (\u0442\u043e\u043b\u044c\u043a\u043e \u043f\u043e\u043b\u044c\u0437\u043e\u0432\u0430\u0442\u0435\u043b\u0438)')),
                ('user_photos', models.PositiveIntegerField(default=0, verbose_name='\u0424\u043e\u0442\u043e\u0433\u0440\u0430\u0444\u0438\u0439 \u0441 \u043f\u043e\u043b\u044c\u0437\u043e\u0432\u0430\u0442\u0435\u043b\u0435\u043c')),
                ('user_videos', models.PositiveIntegerField(default=0, verbose_name='\u0412\u0438\u0434\u0435\u043e\u0437\u0430\u043f\u0438\u0441\u0435\u0439 \u0441 \u043f\u043e\u043b\u044c\u0437\u043e\u0432\u0430\u0442\u0435\u043b\u0435\u043c')),
                ('is_deactivated', models.BooleanField(default=False, db_index=True, verbose_name='\u0414\u0435\u0430\u043a\u0442\u0438\u0432\u0438\u0440\u043e\u0432\u0430\u043d?')),
                ('has_avatar', models.BooleanField(default=True, db_index=True, verbose_name='\u0415\u0441\u0442\u044c \u0430\u0432\u0430\u0442\u0430\u0440?')),
                ('city', models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, to='vkontakte_places.City', null=True)),
                ('country', models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, to='vkontakte_places.Country', null=True)),
                ('friends_users', models.ManyToManyField(related_name='followers_users', to='vkontakte_users.User')),
            ],
            options={
                'verbose_name': '\u041f\u043e\u043b\u044c\u0437\u043e\u0432\u0430\u0442\u0435\u043b\u044c \u0412\u043a\u043e\u043d\u0442\u0430\u043a\u0442\u0435',
                'verbose_name_plural': '\u041f\u043e\u043b\u044c\u0437\u043e\u0432\u0430\u0442\u0435\u043b\u0438 \u0412\u043a\u043e\u043d\u0442\u0430\u043a\u0442\u0435',
            },
            bases=(vkontakte_api.models.RemoteIdModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='UserRelative',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.CharField(max_length=20, verbose_name='\u0422\u0438\u043f \u0440\u043e\u0434\u0441\u0442\u0432\u0435\u043d\u043d\u043e\u0439 \u0441\u0432\u044f\u0437\u0438', choices=[(b'grandchild', '\u0432\u043d\u0443\u043a/\u0432\u043d\u0443\u0447\u043a\u0430'), (b'grandparent', '\u0434\u0435\u0434\u0443\u0448\u043a\u0430/\u0431\u0430\u0431\u0443\u0448\u043a\u0430'), (b'child', '\u0441\u044b\u043d/\u0434\u043e\u0447\u043a\u0430'), (b'sibling', '\u0431\u0440\u0430\u0442/\u0441\u0435\u0441\u0442\u0440\u0430'), (b'parent', '\u043c\u0430\u043c\u0430/\u043f\u0430\u043f\u0430')])),
                ('user1', models.ForeignKey(related_name='user_relatives1', to='vkontakte_users.User')),
                ('user2', models.ForeignKey(related_name='user_relatives2', to='vkontakte_users.User')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import annoying.fields


class Migration(migrations.Migration):

    dependencies = [
        ('vkontakte_users', '0002_auto_20160213_0238'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='schools',
            field=annoying.fields.JSONField(null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='universities',
            field=annoying.fields.JSONField(null=True, blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('vkontakte_users', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='instagram',
            field=models.CharField(default='', max_length=30),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='user',
            name='is_banned',
            field=models.BooleanField(default=False, verbose_name='\u0417\u0430\u0431\u0430\u043d\u0435\u043d'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='user',
            name='is_deleted',
            field=models.BooleanField(default=False, verbose_name='\u0423\u0434\u0430\u043b\u0435\u043d'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='user',
            name='nickname',
            field=models.CharField(default='', max_length=32),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='user',
            name='schools',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='user',
            name='universities',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='user',
            name='bdate',
            field=models.CharField(max_length=10),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='facebook',
            field=models.CharField(max_length=18),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='facebook_name',
            field=models.CharField(max_length=50),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='faculty_name',
            field=models.CharField(max_length=255),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='first_name',
            field=models.CharField(max_length=32),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='has_avatar',
            field=models.BooleanField(default=True, db_index=True, verbose_name='\u0415\u0441\u0442\u044c \u0430\u0432\u0430\u0442\u0430\u0440'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='home_phone',
            field=models.CharField(max_length=24),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='is_deactivated',
            field=models.BooleanField(default=False, db_index=True, verbose_name='\u0414\u0435\u0430\u043a\u0442\u0438\u0432\u0438\u0440\u043e\u0432\u0430\u043d'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='last_name',
            field=models.CharField(max_length=32),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='livejournal',
            field=models.CharField(max_length=31),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='mobile_phone',
            field=models.CharField(max_length=24),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='screen_name',
            field=models.CharField(max_length=32, db_index=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='skype',
            field=models.CharField(max_length=32),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='twitter',
            field=models.CharField(max_length=15),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='user',
            name='university_name',
            field=models.CharField(max_length=255),
            preserve_default=True,
        ),
    ]
